docs.rs failed to build sp1-recursion-compiler-6.1.0
Please check the build logs for more information.
See Builds for ideas on how to fix a failed build, or Metadata for how to configure docs.rs builds.
If you believe this is docs.rs' fault, open an issue.
Please check the build logs for more information.
See Builds for ideas on how to fix a failed build, or Metadata for how to configure docs.rs builds.
If you believe this is docs.rs' fault, open an issue.
Visit the last successful build:
sp1-recursion-compiler-6.0.2
sp1-recursion-compiler
Compiler for SP1 recursion circuits.
Compiles recursion programs into circuit representations for proof aggregation.
Part of SP1, a performant zkVM.