pub struct AsmCompiler { /* private fields */ }Expand description
The backend for the circuit compiler.
Implementations
Sourceimpl AsmCompilerwhere
SP1Field: PrimeField64,
impl AsmCompilerwhere
SP1Field: PrimeField64,
Sourcepub fn alloc(next_addr: &mut SP1Field) -> Address<SP1Field>
pub fn alloc(next_addr: &mut SP1Field) -> Address<SP1Field>
Allocate a fresh address. Checks that the address space is not full.
Sourcepub fn read_ghost_vaddr(&mut self, vaddr: usize) -> Address<SP1Field>
pub fn read_ghost_vaddr(&mut self, vaddr: usize) -> Address<SP1Field>
Map fp to its existing address without changing its mult.
Ensures that fp has already been assigned an address.
Sourcepub fn read_vaddr(&mut self, vaddr: usize) -> Address<SP1Field>
pub fn read_vaddr(&mut self, vaddr: usize) -> Address<SP1Field>
Map fp to its existing address and increment its mult.
Ensures that fp has already been assigned an address.
pub fn read_vaddr_internal( &mut self, vaddr: usize, increment_mult: bool, ) -> Address<SP1Field>
Sourcepub fn write_fp(&mut self, vaddr: usize) -> Address<SP1Field>
pub fn write_fp(&mut self, vaddr: usize) -> Address<SP1Field>
Map fp to a fresh address and initialize the mult to 0.
Ensures that fp has not already been written to.
Sourcepub fn read_addr(&mut self, addr: Address<SP1Field>) -> &mut SP1Field
pub fn read_addr(&mut self, addr: Address<SP1Field>) -> &mut SP1Field
Increment the existing mult associated with addr.
Ensures that addr has already been assigned a mult.
Sourcepub fn read_ghost_addr(&mut self, addr: Address<SP1Field>) -> &mut SP1Field
pub fn read_ghost_addr(&mut self, addr: Address<SP1Field>) -> &mut SP1Field
Retrieves mult associated with addr.
Ensures that addr has already been assigned a mult.
Sourcepub fn write_addr(&mut self, addr: Address<SP1Field>) -> &mut SP1Field
pub fn write_addr(&mut self, addr: Address<SP1Field>) -> &mut SP1Field
Associate a mult of zero with addr.
Ensures that addr has not already been written to.
Sourcepub fn read_const(
&mut self,
imm: Imm<SP1Field, SP1ExtensionField>,
) -> Address<SP1Field>
pub fn read_const( &mut self, imm: Imm<SP1Field, SP1ExtensionField>, ) -> Address<SP1Field>
Read a constant (a.k.a. immediate).
Increments the mult, first creating an entry if it does not yet exist.
Sourcepub fn read_ghost_const(
&mut self,
imm: Imm<SP1Field, SP1ExtensionField>,
) -> Address<SP1Field>
pub fn read_ghost_const( &mut self, imm: Imm<SP1Field, SP1ExtensionField>, ) -> Address<SP1Field>
Read a constant (a.k.a. immediate).
Does not increment the mult. Creates an entry if it does not yet exist.
Sourcepub fn compile_one<C: Config<N = SP1Field>>(
&mut self,
ir_instr: DslIr<C>,
consumer: impl FnMut(Result<Instruction<SP1Field>, CompileOneErr<C>>),
)
pub fn compile_one<C: Config<N = SP1Field>>( &mut self, ir_instr: DslIr<C>, consumer: impl FnMut(Result<Instruction<SP1Field>, CompileOneErr<C>>), )
Compiles one instruction, passing one or more instructions to consumer.
We do not simply return a Vec for performance reasons — results would be immediately fed
to flat_map, so we employ fusion/deforestation to eliminate intermediate data structures.
Sourcepub fn compile<C: Config<N = SP1Field>>(
&mut self,
program: DslIrProgram<C>,
) -> RecursionProgram<SP1Field>
pub fn compile<C: Config<N = SP1Field>>( &mut self, program: DslIrProgram<C>, ) -> RecursionProgram<SP1Field>
Compile a DslIrProgram that is definitionally assumed to be well-formed.
Returns a well-formed program.
Sourcepub fn compile_inner<C: Config<N = SP1Field>>(
&mut self,
root_block: DslIrBlock<C>,
) -> RootProgram<SP1Field>
pub fn compile_inner<C: Config<N = SP1Field>>( &mut self, root_block: DslIrBlock<C>, ) -> RootProgram<SP1Field>
Compile a root DslIrBlock that has not necessarily been validated.
Returns a program that may be ill-formed.