Builder

Struct Builder 

pub struct Builder<C: Config> {
    pub poseidon2_constants: Vec<Ext<SP1Field, SP1ExtensionField>>,
    pub program_type: RecursionProgramType,
    /* private fields */
}
A builder for the DSL.

Can compile to both assembly and a set of constraints.

Fields

poseidon2_constants: Vec<Ext<SP1Field, SP1ExtensionField>>program_type: RecursionProgramType

Implementations

Source

impl<C: Config> Builder<C>

pub fn num2bits_v_circuit( &mut self, num: Var<C::N>, bits: usize, ) -> Vec<Var<C::N>>

Converts a variable to bits inside a circuit.

pub fn num2bits_f_circuit(&mut self, num: Felt<SP1Field>) -> Vec<Var<C::N>>

Converts a felt to bits inside a circuit.

pub fn bits2num_v_circuit(&mut self, bits: &[Var<C::N>]) -> Var<C::N>

Convert bits to a variable inside a circuit.

Source

impl<C: Config> Builder<C>

pub fn new(program_type: RecursionProgramType) -> Self

pub fn new_sub_builder( variable_count: u32, nb_public_values: Option<Var<C::N>>, p2_hash_num: Var<C::N>, debug: bool, program_type: RecursionProgramType, ) -> Self

Creates a new builder with a given number of counts for each type.

pub fn sub_builder(&self) -> Self

Convenience function for creating a new sub builder.

pub fn push_op(&mut self, op: DslIr<C>)

Pushes an operation to the builder.

pub fn extend_ops(&mut self, ops: impl IntoIterator<Item = DslIr<C>>)

pub fn push_backtrace(&mut self)

pub fn push_traced_op(&mut self, op: DslIr<C>)

Pushes an operation to the builder and records a trace if the “debug” feature is enabled.

pub fn variable_count(&self) -> u32

pub fn into_operations(self) -> Vec<DslIr<C>>

pub fn into_root_block(self) -> DslIrBlock<C>

pub fn get_mut_operations(&mut self) -> &mut Vec<DslIr<C>>

Get a mutable reference to the list of operations. Can be used for adjusting evaluation order using the utility functions from std::mem.

One use case is to move “lazy” evaluation out of a parallel context.

pub fn uninit<V: Variable<C>>(&mut self) -> V

Creates an uninitialized variable.

pub fn eval<V: Variable<C>, E: Into<V::Expression>>(&mut self, expr: E) -> V

Evaluates an expression and returns a variable.

pub fn constant<V: FromConstant<C>>(&mut self, value: V::Constant) -> V

Evaluates a constant expression and returns a variable.

pub fn assign<V: Variable<C>, E: Into<V::Expression>>( &mut self, dst: V, expr: E, )

Assigns an expression to a variable.

pub fn assert_eq<V: Variable<C>>( &mut self, lhs: impl Into<V::Expression>, rhs: impl Into<V::Expression>, )

Asserts that two expressions are equal.

pub fn assert_ne<V: Variable<C>>( &mut self, lhs: impl Into<V::Expression>, rhs: impl Into<V::Expression>, )

Asserts that two expressions are not equal.

pub fn assert_var_eq<LhsExpr: Into<SymbolicVar<C::N>>, RhsExpr: Into<SymbolicVar<C::N>>>( &mut self, lhs: LhsExpr, rhs: RhsExpr, )

Assert that two vars are equal.

pub fn assert_var_ne<LhsExpr: Into<SymbolicVar<C::N>>, RhsExpr: Into<SymbolicVar<C::N>>>( &mut self, lhs: LhsExpr, rhs: RhsExpr, )

Assert that two vars are not equal.

pub fn assert_felt_eq<LhsExpr: Into<SymbolicFelt<SP1Field>>, RhsExpr: Into<SymbolicFelt<SP1Field>>>( &mut self, lhs: LhsExpr, rhs: RhsExpr, )

Assert that two felts are equal.

pub fn assert_felt_ne<LhsExpr: Into<SymbolicFelt<SP1Field>>, RhsExpr: Into<SymbolicFelt<SP1Field>>>( &mut self, lhs: LhsExpr, rhs: RhsExpr, )

Assert that two felts are not equal.

pub fn assert_ext_eq<LhsExpr: Into<SymbolicExt<SP1Field, SP1ExtensionField>>, RhsExpr: Into<SymbolicExt<SP1Field, SP1ExtensionField>>>( &mut self, lhs: LhsExpr, rhs: RhsExpr, )

Assert that two exts are equal.

pub fn assert_ext_ne<LhsExpr: Into<SymbolicExt<SP1Field, SP1ExtensionField>>, RhsExpr: Into<SymbolicExt<SP1Field, SP1ExtensionField>>>( &mut self, lhs: LhsExpr, rhs: RhsExpr, )

Assert that two exts are not equal.

pub fn print_debug(&mut self, val: usize)

pub fn print_v(&mut self, dst: Var<C::N>)

Print a variable.

pub fn print_f(&mut self, dst: Felt<SP1Field>)

Print a felt.

pub fn print_e(&mut self, dst: Ext<SP1Field, SP1ExtensionField>)

Print an ext.

pub fn witness_var(&mut self) -> Var<C::N>

pub fn witness_felt(&mut self) -> Felt<SP1Field>

pub fn witness_ext(&mut self) -> Ext<SP1Field, SP1ExtensionField>

pub fn error(&mut self)

Throws an error.

pub fn commit_public_value(&mut self, val: Felt<SP1Field>)

Register and commits a felt as public value. This value will be constrained when verified.

pub fn commit_vkey_hash_circuit(&mut self, var: Var<C::N>)

pub fn commit_committed_values_digest_circuit(&mut self, var: Var<C::N>)

pub fn commit_exit_code_circuit(&mut self, var: Var<C::N>)

pub fn commit_proof_nonce_circuit(&mut self, var: Var<C::N>)

pub fn commit_vk_root_circuit(&mut self, var: Var<C::N>)

pub fn reduce_e(&mut self, ext: Ext<SP1Field, SP1ExtensionField>)

pub fn felt2var_circuit(&mut self, felt: Felt<SP1Field>) -> Var<C::N>

Source

impl<C: Config> Builder<C>

pub fn generator(&mut self) -> Felt<SP1Field>

The generator for the field.

Reference: [p3_koala_bear::KoalaBear]

pub fn select_v( &mut self, cond: Var<C::N>, a: Var<C::N>, b: Var<C::N>, ) -> Var<C::N>

Select a variable based on a condition.

pub fn select_f( &mut self, cond: Var<C::N>, a: Felt<SP1Field>, b: Felt<SP1Field>, ) -> Felt<SP1Field>

Select a felt based on a condition.

pub fn select_ef( &mut self, cond: Var<C::N>, a: Ext<SP1Field, SP1ExtensionField>, b: Ext<SP1Field, SP1ExtensionField>, ) -> Ext<SP1Field, SP1ExtensionField>

Select an extension based on a condition.

pub fn exp_power_of_2<V: Variable<C>, E: Into<V::Expression>>( &mut self, e: E, power_log: usize, ) -> V

Exponentiates a variable to a power of two.

pub fn exp_f_bits( &mut self, x: Felt<SP1Field>, power_bits: Vec<Var<C::N>>, ) -> Felt<SP1Field>

Exponentiates a felt to a list of bits in little endian.

pub fn exp_e_bits( &mut self, x: Ext<SP1Field, SP1ExtensionField>, power_bits: Vec<Var<C::N>>, ) -> Ext<SP1Field, SP1ExtensionField>

Exponentiates a extension to a list of bits in little endian.

pub fn exp_power_of_2_v_circuit<V>( &mut self, base: impl Into<V::Expression>, power_log: usize, ) -> V
where V: Copy + Mul<Output = V::Expression> + Variable<C>,

Exponentiates a variable to a list of bits in little endian inside a circuit.

pub fn ext_from_base_slice( &mut self, arr: &[Felt<SP1Field>], ) -> Ext<SP1Field, SP1ExtensionField>

Creates an ext from a slice of felts.

pub fn felts2ext( &mut self, felts: &[Felt<SP1Field>], ) -> Ext<SP1Field, SP1ExtensionField>

pub fn ext2felt_circuit( &mut self, value: Ext<SP1Field, SP1ExtensionField>, ) -> [Felt<SP1Field>; 4]

Converts an ext to a slice of felts inside a circuit.